function Polygon = CEB_PolygonCurveEvolution(Contour, Center, NumP)
% 
% CEB_PolygonCurveEvolution
% CEB_PolygonCurveEvolution(Contour)
%
% CEB_PolygonCurveEvolution: Evole contour points of x, y coordinates to a simple polygon
% 
% Input:        A Nx2 matrix that contains pairs of x and y coordinates of a closed boundary, 
%               center of the contour, and the number of points to keep 
% Output:       A Nx2 matrix that contains pairs of x and y coordinates of the polygon vertics
%
% Example:      Polygon = CEB_PolygonCurveEvolution(Contour);
% 
% Notes:
%
% Revision history (Date/author/comment):
%
% Created by:           D. J. Lee
% Creation Date:        June 24, 2002
% This version:         1.0
% Date of this version: June 25, 2002

% Modified by Zhiyun on March 2013 (remove plot functions)
%----------------------------------------------------

[r, c] = size(Contour);
% Find data range for better display
max_x = max(Contour(:,1))+10;
min_x = min(Contour(:,1))-10;
max_y = max(Contour(:,2))+10;
min_y = min(Contour(:,2))-10;

% Remove duplicated points  Notes: Spline output has quite a few duplicate points (Same x and y)
Current(1,:)=Contour(1,:);   
iNum=1;
for i=1:r-1     % remove the last point that was appended for closed contour
    if (Current(iNum,:) ~= Contour(i,:))
        iNum=iNum+1;
        Current(iNum,:) = Contour(i,:);
    end
end

% Repeat the evolution till reach desired number of data points (NumP)
if(iNum>NumP)
    Iteration = iNum-NumP+1;
else
    Iteration = 3;
end

for j=1:Iteration
    
    % Calculate the turn angle (TA) for the first pixel as initial value
    Min=CEB_TurnAngle(Current(iNum,:), Current(1,:), Current(2,:), Center);   
    iMin=1;   
    % Find the smallest Relevence Measure (RM)
    for i=2:iNum
        if (i == iNum)
            Last = 1;
        else
            Last = i+1;
        end
        K=CEB_TurnAngle(Current(i-1,:), Current(i,:), Current(Last,:), Center);
        if (K >= 0.0)
            if (K < Min) 
                Min=K;
                iMin=i;
            end
        end    
    end
    % Remove the point with the smallest RM and shift up the remainder points by one position
    for k=iMin:iNum-1
        Current(k,:)=Current(k+1,:);
    end 
    Current(iNum,:)=Current(1,:);   % Append the first point to the end for displaying closed contour
    Polygon=zeros(iNum, 2);         % Create a new buffer with different size each time for display
    for k=1:iNum
        Polygon(k,:)=Current(k,:);
    end
    iNum=iNum-1;    % One point has been removed
    
end

% Calculate the turn angle using the following example
% A=(1,2)  B=(3,2)  C=(4,5)  T=angle between AC and AB
% Vector direction Vac=C-A=(3,3) and Vab=B-A=(2, 0)
% cos(TA) = (Vac . Vab)/||Vac|| x ||Vab|| (. is vector product)
% cos(TA)=(6+0)/(18^0.5 x 4^0.5) -> T=45 degrees
% Relevence should always be > 0, Use -1 to indicate that points are overlapped
function Relevence=CEB_TurnAngle(P_Minus1, P, P_Plus1, Center)
AC=P_Minus1-P;
AB=P_Plus1-P;
LC=(AC(1)*AC(1)+AC(2)*AC(2))^0.5;
LB=(AB(1)*AB(1)+AB(2)*AB(2))^0.5;
Relevence = -1.0;
if (LC~=0 & LB~=0)
   M=(P_Minus1+P_Plus1)/2; % Mid point of B & C           
   CenterToA=Center-P;
   % Distance from A to the center
   LCenterToA=(CenterToA(1)*CenterToA(1)+CenterToA(2)*CenterToA(2))^0.5;
   % Distance from mid point to the center
   CenterToM=Center-M;
   LCenterToM=(CenterToM(1)*CenterToM(1)+CenterToM(2)*CenterToM(2))^0.5;
   % Turn Angle Equation
   TT=(AC(1)*AB(1)+AC(2)*AB(2))/(LC*LB);        
   TA=acos(TT);
   % If A is farther from center than mid point of B&C, turn angle must be > 180
   if (LCenterToA > LCenterToM) 
      TA=2.0*pi-TA;
   end
   % Relevence Measure
   Relevence=TA*LC*LB/(LC+LB);
end