function Mask = LungSegment(fname,ModelPath)

%-----------------------------------------------------------------------------
%
%   mask = LungSegment_module(fname,Param) performs lung segmentation on CXRs.
%   Algorithm (1)reads the patient X-ray, (2)register the model X-rays to the
%   patient X-ray, (3)computes the similarities (transformation map),
%   (4) apply the transformation map to the model masks to obtain
%   approximate patient lung mask. 
%
%
%   Sema Candemir
%   sema.candemir@nih.gov
%   National Library of Medicine, National Institutes of Health
%   8600 Rockville Pike, Bethesda, MD 20894, USA

%   Reference:
%      Candemir, Sema, et al., "Lung segmentation in chest radiographs using 
%      anatomical atlases with nonrigid registration.",
%      Medical Imaging, IEEE Transactions on 33.2 (2014): 577-590.
%
%-----------------------------------------------------------------------------

    % Parameters
    Param.NofMask = 5;
    Param.SIFTflowparam.cellsize = 3;
    Param.SIFTflowparam.gridspacing = 1;
    Param.SIFTflowparam.alpha = 2*90;
    Param.SIFTflowparam.d = 40*255;
    Param.SIFTflowparam.gamma = 0.001;
    Param.SIFTflowparam.nlevels = 4;
    Param.SIFTflowparam.wsize = 3;
    Param.SIFTflowparam.topwsize = 10;
    Param.SIFTflowparam.nTopIterations = 100;
    Param.SIFTflowparam.nIterations = 60;
    Param.ImPath = ModelPath;
    Param.GTPath = ModelPath;
    Param.MaskThreshold = 0.6;
    Param.BorderSmoothing = 100;
    
    % Read Image
    fprintf('Reading patient X-ray...\n')
    Img = read_Xray(fname);

    % Resize Image (in order to perform the registration in a smaller size image)
    OrigSize = size(Img);
    Img = imresize(Img, [256 NaN],'bilinear');
    
    
    % Pre-processing: crop the black region around the boundary
    [Img crop] = f_removeBlackBorders(Img);
       
    % Pre-processing: histogram equalization
    Img = double(adapthisteq(Img));
    % Img = double(histeq(Img));

    % Lung Model Computation...
    Mask = RegisterModel(Img,Param);
    
    % Smooth the lung mask... (replacement of the graph cut section in the paper)
    Mask(Mask<Param.MaskThreshold) = 0;
    Mask(Mask>=Param.MaskThreshold) = 1;
    Mask = SmoothBoundary(Mask,Param.BorderSmoothing);
    
    % Overlay the computed lung boundary on patient X-ray
    Edges_S = edge(Mask,'canny');
    [cols, rows] = find(Edges_S ==1);
    imshow(Img,[]),hold on
    plot(rows,cols,'r.','MarkerSize',3)
    
    % Save the overlay image
    [a,b,ext] = fileparts(fname);
    fname = sprintf('%s_overlay.png',b);
    print('-dpng',fname);
    png = imread(fname);
    m = min(min(png), [], 3) ~= 255;
    png = png(:, m, :);
    m = min(min(png, [], 2), [], 3) ~= 255;
    png = png(m, :, :);
    png = imresize(png, [size(Img,1) size(Img,2)],'bilinear');
    imwrite(png, fname, 'png');
    close all
    
    % resize the lung mask to the original size of patient X-ray...
    Mask = imresize(Mask,OrigSize,'bilinear'); % resize it to original size
    
end