function LungMask = RegisterModel(Img,Param)

%-----------------------------------------------------------------------------
%
%   LungModel = RegisterModel(Img,Param) registers the model X-rays to the
%   patient X-ray and computes the similarities (transformation map)
%   between CXRs. Then, it applies the transformation map to the model masks to obtain
%   approximate patient lung mask. Some part of this code employes SIFTflow
%   algorithm. Please visit "http://people.csail.mit.edu/celiu/SIFTflow/"  
%   for mexDenseSIFT and SIFTflowc2f functions.
%
%   Sema Candemir
%   sema.candemir@nih.gov
%   National Library of Medicine, National Institutes of Health
%   8600 Rockville Pike, Bethesda, MD 20894, USA

%   References:
%      Candemir, Sema, et al., "Lung segmentation in chest radiographs using 
%      anatomical atlases with nonrigid registration.",
%      Medical Imaging, IEEE Transactions on 33.2 (2014): 577-590.

%      Liu, Ce, et al. "Sift flow: Dense correspondence across scenes and 
%      its applications." Pattern Analysis and Machine Intelligence, IEEE 
%      Transactions on 33.5 (2011): 978-994.

%
%-----------------------------------------------------------------------------

LungMask = zeros(size(Img,1), size(Img,2));

% find the most similar X-rays in the model set...
maxIndex = FindSimilarCXR(Img,Param);
% maxIndex = maxIndex(2:end); (only for JSRT)

ext = {'*.tif','*.dcm', '*.png', '*.bmp','*.jpg'};
imlist = [];
for i = 1:length(ext)
    list = dir([Param.ImPath ext{i}]);
    imlist = [imlist; list];
end

% computes the SIFT matrix for the patient CXRs
sift1 = mexDenseSIFT(Img,Param.SIFTflowparam.cellsize,Param.SIFTflowparam.gridspacing);

for index = maxIndex

    fname_model = imlist(index).name;
    [ModelImg, ModelMask] = read_Xray_Mask(fname_model,Param);
    ModelImg = imresize(ModelImg,size(Img),'bilinear'); % Resize model Img
    ModelMask = imresize(ModelMask,size(Img),'bilinear'); % Resize model Mask
    % ModelImg = double(histeq(ModelImg));  %% histogram equalization
    ModelImg = double(adapthisteq(ModelImg));  %% histogram equalization

    % SIFT matrix for each model CXRs...
    sift2 = mexDenseSIFT(ModelImg,Param.SIFTflowparam.cellsize,Param.SIFTflowparam.gridspacing);
    
    % compute the correspondences between two sift matrix of CXRs...
    [vx,vy,~] = SIFTflowc2f(sift1,sift2,Param.SIFTflowparam);
    
    % Apply transformation map to model mask
    ModelMask = warpImage(ModelMask,vx,vy);
    
    % reduce the noise which occurs at the borders due to registration process
    ModelMask = f_RemoveSmallSegments(ModelMask);

    % Avg Lung Shape
    LungMask = LungMask + ModelMask;
end

LungMask = LungMask/length(maxIndex);
