function Img = read_Xray(fname)

%-----------------------------------------------------------------------------
%
%   Img = read_Xray(fname) reads the CXRs. The supported image formats are 
%   dcm, tif, tiff, png, bmp, jpg
%
%   Stefan Jaeger,Sema Candemir
%   email: stefan.jaeger@nih.gov,sema.candemir@nih.gov
%   National Library of Medicine, National Institutes of Health
%   8600 Rockville Pike, Bethesda, MD 20894, USA

%   Reference:
%      Candemir, Sema, et al., "Lung segmentation in chest radiographs using 
%      anatomical atlases with nonrigid registration.",
%      Medical Imaging, IEEE Transactions on 33.2 (2014): 577-590.
%
%-----------------------------------------------------------------------------

    % Read x-ray image
    [~,~,ext] = fileparts(fname);

    if(strcmp(ext,'.dcm'))
        Img = dicomread(fname);
        info = dicominfo(fname);
        Img = double(Img);       
        
        %% the next part is for only Indiana data. 
        if (isfield(info,'WindowWidth'))
            minI = max(0,info.WindowCenter(1)-(info.WindowWidth(1)/2));
            maxI = min(info.WindowCenter(1)+(info.WindowWidth(1)/2),double((2^info.BitDepth)-1));
            Img(Img<minI) = minI;
            Img(Img>maxI) = maxI;
        end
                
    else
        Img = imread(fname);
        info = imfinfo(fname);
        
        if (isfield(info,'ColorType') && strcmp(info.ColorType,'indexed'))
            Img = ind2gray(Img,info.Colormap);
        end
        
        if (size(Img,3)>1)
            Img = rgb2gray(Img);  % 3D image detected
        end
        
        Img = double(Img);
    end
    
    
    %% Normalize the image betwen 0-1
    max_int = 1; 
    A = [min(min(Img)) max_int; max(max(Img)) max_int];
    b = [0;max_int];
    par = A\b;
    Img = (Img .* par(1)) + par(2);
    % handle potential numerical problems
    Img(Img<0)=0;
    Img(Img>max_int)=max_int;
    

    if (isfield(info,'PhotometricInterpretation') && strcmp(info.PhotometricInterpretation,'MONOCHROME1'))
        Img = max_int-Img;
    end
    
end